// module mutex.cpp
// Private Copyright Dr.E.Huckert 2-2000
// mutexes for Windows and UNIX (POSIX mutexes)
// Note: FILE_Mutex is not implemented/tested for WINDOWS!

#include "mutex.h"

// -------------------------------------------------------------
// class implementation for Mutex
// Constructor
Mutex::Mutex(const char *name)
{
int flag, n, m, ret;
  logging = 0;
  m       = 0;
  for (n=0; name[n] != 0; n++)
  {
    if (m < (sizeof(this->mut_name) -1))
      this->mut_name[m++] = name[n];
  }
  this->mut_name[m] = 0;
#ifdef WIN32
  mutex = CreateMutex(NULL, FALSE, name);
#else
  pthread_mutexattr_t mu_attr;
  ret = pthread_mutexattr_init(&mu_attr);
  ret = pthread_mutex_init(&(this->mutex), &mu_attr); 
#endif
};

// -------------------------------------------------------------
// Destructor
Mutex::~Mutex()
{
  this->unlock();
#ifdef WIN32
  CloseHandle(this->mutex); 
#else
  pthread_mutex_destroy(&(this->mutex));
#endif
  //if (this->logging > 1)
  //  cout << "\tMutex::~Mutex():" << mut_name << endl;
}   // end Mutex::~Mutex()

// -------------------------------------------------------------
// waits until an existing lock is removed
long Mutex::setlock()
{
long ret;
  //if (this->logging > 1)
  //  cout << "\tMutex::setlock():" << mut_name 
  //       << " thread=" << this->get_thread() << endl;
  ret = 0;
#ifdef WIN32
    ret = WaitForSingleObject(this->mutex, INFINITE);
#else
    ret = pthread_mutex_lock(&(this->mutex));
#endif
  return ret;   
}   // end Mutex::setlock()

// -------------------------------------------------------------
// remove a lock 
long Mutex::unlock()
{
long ret;
  //if (this->logging > 1)
  //  cout << "\tMutex::unlock():" << mut_name 
  //       << " thread=" << this->get_thread() << endl;
  ret = 0;
#ifdef WIN32
    ret = ReleaseMutex(this->mutex);
#else
    pthread_mutex_unlock(&(this->mutex));
#endif
  return ret;   
}   // end Mutex::unlock()

// -------------------------------------------------------------
void Mutex::setLogging(int flag)
{
  logging = flag;
}

// -------------------------------------------------------------
long Mutex::get_thread()
{
#ifdef WIN32
  return (long)GetCurrentThread();
#else
  return (long)pthread_self();
#endif
}

// -------------------------------------------------------------
FILE_Mutex::FILE_Mutex(int file_id)
{
  this->fd = file_id;
}   // end FILE_Mutex::FILE_Mutex()

// -------------------------------------------------------------
// Destructor
FILE_Mutex::~FILE_Mutex()
{
}

// -------------------------------------------------------------
// mutex lock based on an exclusive lock via fcntl() in Linux
// Returns: 0 if the lock is acquired, else < 0
// Returns immediately with < 0 if the lock cannot be acquired!
int FILE_Mutex::setlock()
{
int ret = -1;
#ifndef WIN32
struct flock flck;
  //if (this->logging > 1)
  //  cout << "\tFILE_Mutex::setlock():" << fd << endl;
  flck.l_type   = F_WRLCK;
  flck.l_whence = SEEK_SET;
  flck.l_start  = 0L;
  flck.l_len    = 0L;
  ret = fcntl(this->fd, F_SETLK, &flck);  // was: F_SETLKW
#endif
  return ret;
}   // end FILE_Mutex::setlock()

// -------------------------------------------------------------
// Returns: 0 if the lock is acquired, else < 0
// Waits until the lock is acquired!
int FILE_Mutex::setlockWait()
{
int ret = -1;
#ifndef WIN32
struct flock flck;
  //if (this->logging > 1)
  //  cout << "\tFILE_Mutex::setlock():" << fd << endl;
  flck.l_type   = F_WRLCK;
  flck.l_whence = SEEK_SET;
  flck.l_start  = 0L;
  flck.l_len    = 0L;
  ret = fcntl(this->fd, F_SETLKW, &flck);  
#endif
  return ret;
}   // end FILE_Mutex::setlockWait()

// -------------------------------------------------------------
// mutex unlock based on file lock via fcntl() 
// Returns: 0 if the lock is released, else < 0
int FILE_Mutex::unlock()
{
int ret = -1;
#ifndef WIN32
struct flock fl;
  //if (this->logging > 1)
  //  cout << "\tFILE_Mutex::unlock():" << fd << endl;
  fl.l_type     = F_UNLCK;
  fl.l_whence   = SEEK_SET;
  fl.l_start    = 0L;
  fl.l_len      = 0L;
  ret = fcntl(this->fd, F_SETLK, &fl);   // was: F_SETLKW
#endif
  return ret;
}   // end FILE_Mutex::Unlock()

// -------------------------------------------------------------
void FILE_Mutex::setLogging(int flag)
{
  logging = flag;
}

